/*
 * Copyright (c) 2014-2015,2017 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *					Anthony Mallet on Wed Feb 19 2014
 */
#ifndef H_OPTITRACK_NATNET
#define H_OPTITRACK_NATNET

#include <sys/time.h>
#include <stdint.h>

typedef struct natnet_context *natnet_context_s;

enum natnet_event {
  NATNET_EVERR = -1,
  NATNET_EVCMD = 1,
  NATNET_EVDAT = 2
};

enum natnet_msg {
  NATNET_MSGERR = -1,
  NATNET_MSGNONE = 0,
  NATNET_MSGPING,
  NATNET_MSGMODELDEF,
  NATNET_MSGFRAMEOFDATA
};

struct natnet_anonmset {
  uint32_t n;			/* number of markers */
  const struct __attribute__ ((__packed__)) {
    float x, y, z;
  } *pos;			/* markers position */
};

struct natnet_msetdescr {
  char *name;			/* model name */
  char safename[64];		/* name without special chars */

  uint32_t nmnames;		/* number of marker names */
  char **mname;			/* marker names */
};

struct natnet_mset {
  const struct natnet_msetdescr *descr;
  const char *name;		/* model name */
  struct natnet_anonmset mset;	/* markers */
};

struct natnet_bodydescr {
  char *name;			/* body name */
  char safename[64];		/* name without special chars */
  uint32_t id;
  uint32_t parent;

  struct {
    float x, y, z;
  } offset;			/* offset */
};

struct natnet_body {
  const struct natnet_bodydescr *descr;
  uint32_t id;
  const struct __attribute__ ((__packed__)) {
    float x, y, z, qx, qy, qz, qw;
  } *pos;			/* body position */

  struct natnet_anonmset mset;	/* associated marker set */
  const uint32_t *mid;		/* marker ids */
  const float *msize;		/* marker sizes */
  float error;			/* mean marker error */
};

struct natnet_frame {
  uint32_t id;			/* frame number */
  float latency;		/* latency */
  uint32_t tcode, tsub;		/* time code */
  struct timeval timestamp;	/* timestamp */

  struct natnet_anonmset aset;	/* anonymous markers set */

  uint32_t nmsets, maxmsets;	/* number of markers sets */
  struct natnet_mset *mset;	/* markers sets */

  uint32_t nbodies, maxbodies;	/* number of rigid bodies */
  struct natnet_body *body;	/* rigid bodies */

  uint8_t refresh;		/* tracked model list has changed */
};

struct natnet_descr {
  uint32_t nmsets, maxmsets;	/* number of marker sets descriptions */
  struct natnet_msetdescr *mset;/* marker sets descriptions */

  uint32_t nbodies, maxbodies;	/* number of bodies descriptions */
  struct natnet_bodydescr *body;/* bodies descriptions */
};

natnet_context_s
	natnet_connect(uint16_t local_port, const char *host,
                const char *host_port, const char *mcast,
                const char *mcast_port, char *emsg, size_t emsgsize);
void	natnet_disconnect(natnet_context_s ctx);

int	natnet_snd_ping(natnet_context_s ctx,
                char *emsg, size_t emsgsize);
int	natnet_snd_modeldef(natnet_context_s ctx,
                char *emsg, size_t emsgsize);

int	natnet_poll(natnet_context_s ctx,
                struct timeval *timeout);
int	natnet_cmd_recv(natnet_context_s ctx, size_t *sz,
                char *emsg, size_t emsgsize);
int	natnet_data_recv(natnet_context_s ctx, size_t *sz,
                char *emsg, size_t emsgsize);

int	natnet_rcv_ping(natnet_context_s ctx,
                char *info, size_t infosize);
const struct natnet_descr *
	natnet_rcv_modeldef(natnet_context_s ctx);
const struct natnet_frame *
	natnet_rcv_frameofdata(natnet_context_s ctx);

#endif /* H_OPTITRACK_NATNET */
